@@ -67,22 +67,25 @@ test_that("differs_from_previous() find the right values and indices", {
   fixed = TRUE
   )
 
-  expect_error(
-    check_differs_from_previous("v3",
-      threshold = 2, return_index = TRUE,
-      factor_conversion_warning = TRUE
-    ),
-    "'col' is factor. 'threshold' must be 'NULL'. Alternatively, convert factor to numeric vector.",
-    fixed = TRUE
-  )
+  #
+  side_effects_12655 <- xpectr::capture_side_effects(check_differs_from_previous("v3",
+                                threshold = 2, return_index = TRUE,
+                                factor_conversion_warning = TRUE
+    ), reset_seed = TRUE)
+  expect_equal(
+    xpectr::strip(side_effects_12655[['error']]),
+    xpectr::strip("1 assertions failed:\n * 'col' is factor. 'threshold' must be 'NULL'. Alternatively, convert factor to numeric\n * vector."),
+    fixed = TRUE)
 
-  expect_error(differs_from_previous(df),
-    "'col' must be specified when 'data' is data frame",
+
+  expect_error(
+    xpectr::strip_msg(differs_from_previous(df)),
+    xpectr::strip("1 assertions failed:\n * 'col' must be specified when 'data' is data.frame."),
     fixed = TRUE
   )
 
   expect_warning(differs_from_previous(v, col = "a"),
-    "'col' not used as 'data' is not a data frame",
+    "'col' is ignored when 'data' is not a data.frame.",
     fixed = TRUE
   )
 
@@ -386,7 +389,7 @@ test_that("fuzz testing input checks for differs_from_previous()", {
   # Testing side effects
   expect_error(
     xpectr::strip_msg(differs_from_previous(data = df, col = NULL, threshold = NULL, direction = "both", return_index = FALSE, include_first = TRUE, handle_na = "ignore", factor_conversion_warning = TRUE)),
-    xpectr::strip("'col' must be specified when 'data' is data frame."),
+    xpectr::strip("1 assertions failed:\n * 'col' must be specified when 'data' is data.frame."),
     fixed = TRUE)
 
   # Testing differs_from_previous(data = df$a, col = "n", ...
@@ -397,7 +400,7 @@ test_that("fuzz testing input checks for differs_from_previous()", {
   side_effects_17920 <- xpectr::capture_side_effects(differs_from_previous(data = df$a, col = "n", threshold = NULL, direction = "both", return_index = FALSE, include_first = TRUE, handle_na = "ignore", factor_conversion_warning = TRUE), reset_seed = TRUE)
   expect_equal(
     xpectr::strip(side_effects_17920[['warnings']]),
-    xpectr::strip(c("'data' is factor. Using as character.", "'col' not used as 'data' is not a data frame")),
+    xpectr::strip(c("'col' is ignored when 'data' is not a data.frame", "'data' is factor. Using as character")),
     fixed = TRUE)
   expect_equal(
     xpectr::strip(side_effects_17920[['messages']]),
@@ -441,7 +444,7 @@ test_that("fuzz testing input checks for differs_from_previous()", {
   side_effects_13400 <- xpectr::capture_side_effects(differs_from_previous(data = df$n, col = "n", threshold = NULL, direction = "both", return_index = FALSE, include_first = TRUE, handle_na = "ignore", factor_conversion_warning = TRUE), reset_seed = TRUE)
   expect_equal(
     xpectr::strip(side_effects_13400[['warnings']]),
-    xpectr::strip("'col' not used as 'data' is not a data frame"),
+    xpectr::strip("'col' is ignored when 'data' is not a data.frame"),
     fixed = TRUE)
   expect_equal(
     xpectr::strip(side_effects_13400[['messages']]),
@@ -850,3 +853,53 @@ test_that("fuzz testing input checks for differs_from_previous()", {
   #
 
 })
+
+
+test_that("differs_from_previous() works with group_by()", {
+  xpectr::set_test_seed(1)
+
+  v <- c("a", "a", "b", "c", "c", "d", "d")
+  df <- data.frame(
+    v = v,
+    v2 = c(1, 1, 1, 2, 2, 2, 3),
+    v3 = factor(c(1, 1, 1, 2, 2, 2, 3)),
+    stringsAsFactors = FALSE
+  )
+
+
+  ## Testing 'xpectr::suppress_mw(df %>% dplyr::group_by(v...'              ####
+  ## Initially generated by xpectr
+  xpectr::set_test_seed(42)
+  # Assigning output
+  output_12655 <- xpectr::suppress_mw(df %>%
+                          dplyr::group_by(v3) %>%
+                          differs_from_previous(col = "v"))
+  # Testing class
+  expect_equal(
+    class(output_12655),
+    "list",
+    fixed = TRUE)
+  # Testing type
+  expect_type(
+    output_12655,
+    type = "list")
+  # Testing values
+  expect_equal(
+    output_12655,
+    list(`1` = "b", `2` = "d", `3` = character(0)))
+  # Testing names
+  expect_equal(
+    names(output_12655),
+    c("1", "2", "3"),
+    fixed = TRUE)
+  # Testing length
+  expect_equal(
+    length(output_12655),
+    3L)
+  # Testing sum of element lengths
+  expect_equal(
+    sum(xpectr::element_lengths(output_12655)),
+    2L)
+  ## Finished testing 'xpectr::suppress_mw(df %>% dplyr::group_by(v...'     ####
+
+})
