@@ -1236,7 +1236,7 @@ test_that("fuzz testing input checks for n_dist method in group_factor()", {
   expect_error(
     xpectr::strip_msg(group_counts(data = df, n = 3, method = "l_starts", starts_col = NULL, force_equal = FALSE, allow_zero = FALSE, descending = FALSE, randomize = FALSE, remove_missing_starts = FALSE)),
     xpectr::strip(paste0("1 assertions failed:\n * when 'method' is 'l_starts' and 'd",
-                         "ata' is a data frame, 'starts_col' must be specified.")),
+                         "ata' is a data.frame, 'starts_col' must be specified.")),
     fixed = TRUE)
 
   # Testing group_counts(data = df, n = 3, method = NULL, ...
@@ -1297,7 +1297,7 @@ test_that("fuzz testing input checks for n_dist method in group_factor()", {
   # Testing side effects
   expect_error(
     xpectr::strip_msg(group_counts(data = df, n = "hej", method = "n_dist", starts_col = NULL, force_equal = FALSE, allow_zero = FALSE, descending = FALSE, randomize = FALSE, remove_missing_starts = FALSE)),
-    xpectr::strip("when 'method' is 'n_dist', 'n' must be numeric scalar."),
+    xpectr::strip("1 assertions failed:\n * 'n' can only be character when method is 'l_starts'."),
     fixed = TRUE)
 
   # Testing group_counts(data = df, n = "auto", method = "...
@@ -1306,7 +1306,7 @@ test_that("fuzz testing input checks for n_dist method in group_factor()", {
   # Testing side effects
   expect_error(
     xpectr::strip_msg(group_counts(data = df, n = "auto", method = "n_dist", starts_col = NULL, force_equal = FALSE, allow_zero = FALSE, descending = FALSE, randomize = FALSE, remove_missing_starts = FALSE)),
-    xpectr::strip("when 'method' is 'n_dist', 'n' must be numeric scalar."),
+    xpectr::strip("1 assertions failed:\n * 'n' can only be character when method is 'l_starts'."),
     fixed = TRUE)
 
   # Testing group_counts(data = df, n = NULL, method = "n_...
@@ -1549,7 +1549,7 @@ test_that("fuzz testing input checks for l_starts method in group_factor()", {
   expect_error(
     xpectr::strip_msg(group_counts(data = df$s, n = "auto", method = "l_starts", starts_col = "s", force_equal = FALSE, allow_zero = FALSE, descending = FALSE, randomize = FALSE, remove_missing_starts = FALSE)),
     xpectr::strip(paste0("1 assertions failed:\n * when 'starts_col' is specified, 'd",
-                         "ata' must be a data frame.")),
+                         "ata' must be a data.frame.")),
     fixed = TRUE)
 
   # Testing group_counts(data = df$c, n = "auto", method =...
@@ -1559,7 +1559,7 @@ test_that("fuzz testing input checks for l_starts method in group_factor()", {
   expect_error(
     xpectr::strip_msg(group_counts(data = df$c, n = "auto", method = "l_starts", starts_col = "s", force_equal = FALSE, allow_zero = FALSE, descending = FALSE, randomize = FALSE, remove_missing_starts = FALSE)),
     xpectr::strip(paste0("1 assertions failed:\n * when 'starts_col' is specified, 'd",
-                         "ata' must be a data frame.")),
+                         "ata' must be a data.frame.")),
     fixed = TRUE)
 
   # Testing group_counts(data = df$f, n = "auto", method =...
@@ -1569,7 +1569,7 @@ test_that("fuzz testing input checks for l_starts method in group_factor()", {
   expect_error(
     xpectr::strip_msg(group_counts(data = df$f, n = "auto", method = "l_starts", starts_col = "s", force_equal = FALSE, allow_zero = FALSE, descending = FALSE, randomize = FALSE, remove_missing_starts = FALSE)),
     xpectr::strip(paste0("1 assertions failed:\n * when 'starts_col' is specified, 'd",
-                         "ata' must be a data frame.")),
+                         "ata' must be a data.frame.")),
     fixed = TRUE)
 
   # Testing group_counts(data = NA, n = "auto", method = "...
@@ -1588,7 +1588,7 @@ test_that("fuzz testing input checks for l_starts method in group_factor()", {
   expect_error(
     xpectr::strip_msg(group_counts(data = 1, n = "auto", method = "l_starts", starts_col = "s", force_equal = FALSE, allow_zero = FALSE, descending = FALSE, randomize = FALSE, remove_missing_starts = FALSE)),
     xpectr::strip(paste0("1 assertions failed:\n * when 'starts_col' is specified, 'd",
-                         "ata' must be a data frame.")),
+                         "ata' must be a data.frame.")),
     fixed = TRUE)
 
   # Testing group_counts(data = NULL, n = "auto", method =...
@@ -1703,7 +1703,7 @@ test_that("fuzz testing input checks for l_starts method in group_factor()", {
   # Testing side effects
   expect_error(
     xpectr::strip_msg(group_counts(data = df, n = "auto", method = "n_dist", starts_col = "s", force_equal = FALSE, allow_zero = FALSE, descending = FALSE, randomize = FALSE, remove_missing_starts = FALSE)),
-    xpectr::strip("when 'method' is 'n_dist', 'n' must be numeric scalar."),
+    xpectr::strip("2 assertions failed:\n * 'n' can only be character when method is 'l_starts'.\n * when method is not 'l_starts', 'starts_col' must be 'NULL'."),
     fixed = TRUE)
 
   # Testing group_counts(data = df, n = "auto", method = N...
@@ -1989,10 +1989,74 @@ test_that("fuzz testing input checks for l_starts method in group_factor()", {
   expect_error(
     xpectr::strip_msg(group_counts(data = df, n = "auto", method = "l_starts", starts_col = NULL, force_equal = FALSE, allow_zero = FALSE, descending = FALSE, randomize = FALSE, remove_missing_starts = FALSE)),
     xpectr::strip(paste0("1 assertions failed:\n * when 'method' is 'l_starts' and 'd",
-                         "ata' is a data frame, 'starts_col' must be specified.")),
+                         "ata' is a data.frame, 'starts_col' must be specified.")),
     fixed = TRUE)
 
   ## Finished testing 'group_counts'                                          ####
   #
 
 })
+
+
+test_that("group_factor() works with group_by()", {
+  xpectr::set_test_seed(42)
+
+  df <- data.frame(
+    "n" = c(1, 2, 3, 4, 2, 1, 5, 2, 1, 9),
+    "s" = c(4, 4, 4, 4, 7, 7, 7, 7, 1, 1),
+    "c" = as.character(c(4, 4, 6, 6, 7, 7, 7, 8, 8, 1)),
+    "f" = as.factor(as.character(c(4, 4, 6, 6, 7, 7, 7, 8, 8, 1))),
+    stringsAsFactors = FALSE
+  )
+
+
+  ## Testing 'xpectr::suppress_mw(df %>% dplyr::group_by(s...'              ####
+  ## Initially generated by xpectr
+  xpectr::set_test_seed(42)
+  # Assigning output
+  output_19148 <- xpectr::suppress_mw(df %>%
+                          dplyr::group_by(s) %>%
+                          group_factor(n = 2))
+  # Testing class
+  expect_equal(
+    class(output_19148),
+    c("tbl_df", "tbl", "data.frame"),
+    fixed = TRUE)
+  # Testing column values
+  expect_equal(
+    output_19148[["s"]],
+    c(4, 4, 4, 4, 7, 7, 7, 7, 1, 1),
+    tolerance = 1e-4)
+  expect_equal(
+    output_19148[[".groups"]],
+    structure(c(1L, 1L, 2L, 2L, 1L, 1L, 2L, 2L, 1L, 2L), .Label = c("1",
+      "2"), class = "factor"))
+  # Testing column names
+  expect_equal(
+    names(output_19148),
+    c("s", ".groups"),
+    fixed = TRUE)
+  # Testing column classes
+  expect_equal(
+    xpectr::element_classes(output_19148),
+    c("numeric", "factor"),
+    fixed = TRUE)
+  # Testing column types
+  expect_equal(
+    xpectr::element_types(output_19148),
+    c("double", "integer"),
+    fixed = TRUE)
+  # Testing dimensions
+  expect_equal(
+    dim(output_19148),
+    c(10L, 2L))
+  # Testing group keys
+  expect_equal(
+    colnames(dplyr::group_keys(output_19148)),
+    character(0),
+    fixed = TRUE)
+  ## Finished testing 'xpectr::suppress_mw(df %>% dplyr::group_by(s...'     ####
+
+
+})
+
